package de.wehner.mediamagpie.conductor.webapp.controller.commands.config;

import org.apache.commons.lang.builder.ReflectionToStringBuilder;
import org.apache.commons.lang.builder.ToStringStyle;
import org.apache.commons.lang3.StringUtils;

import de.wehner.mediamagpie.persistence.entity.properties.S3Configuration;

public class S3ConfigurationCommand extends S3Configuration {

    private final int MAX_ANONYMIZED_LENGTH = 20;
    private final char ANONYM_CHAR = '*';

    public S3ConfigurationCommand(String accessKey, String secretKey) {
        this.accessKey = (accessKey != null) ? accessKey.trim() : null;
        this.secretKey = (secretKey != null) ? secretKey.trim() : null;
    }

    public S3ConfigurationCommand() {
    }

    // TODO rwe: I think this would be a good job for dozer.
    public static S3ConfigurationCommand createCommand(S3Configuration s3Configuration) {
        S3ConfigurationCommand command = new S3ConfigurationCommand();
        command.accessKey = s3Configuration.getAccessKey();
        command.secretKey = s3Configuration.getSecretKey();
        command.syncToS3 = s3Configuration.isSyncToS3();
        return command;
    }

    public String getAnonymizedSecretKey() {
        if (StringUtils.isEmpty(secretKey)) {
            return "<not set>";
        }

        int length = secretKey.length();
        if (length <= 3) {
            return StringUtils.repeat(ANONYM_CHAR, length);
        }
        String clearPart = StringUtils.right(secretKey, Math.min(3, length - 3));
        length = Math.min(MAX_ANONYMIZED_LENGTH, length);
        return StringUtils.repeat(ANONYM_CHAR, length - clearPart.length()) + clearPart;
    }

    @Override
    public String toString() {
        return ReflectionToStringBuilder.toString(this, ToStringStyle.SHORT_PREFIX_STYLE);
    }

}
